/*
 Small Programmable Power Supply
 Version: 0.5
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/09/2013
 Last Modified: 18/11/2013

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define REF1_1V ((1<<REFS1) | (0<<REFS0))
#define REFVCC ((0<<REFS1) | (0<<REFS0))

#define dpot_latch PB1
#define dpot_latch_high	PORTB |=  (1<<PB1);
#define dpot_latch_low	PORTB &= ~(1<<PB1);
#define latch PB2
#define latch_high	PORTB |=  (1<<PB2);
#define latch_low		PORTB &= ~(1<<PB2);

#define PROG_BUTTON_1 PA1
#define PROG_BUTTON_2 PB0
#define PROG_BUTTON_3 PA7
#define PROG_BUTTON_SET PA6

// LED A, B, C pin map to 595 Q0-7 pins
#define NA 7
#define NB 0
#define NC 2
#define ND 4
#define NE 5
#define NF 6
#define NG 1
#define NDP 3

// Button LEDs / GNDs
#define LED1 (1<<6)
#define LED2 (1<<5)
#define LED3 (1<<4)

// Array elements
#define DPOTVAL 0
#define ADCVAL 1

// EEPROM locations
#define BUTTON_START_LOCATION 	0
#define BUTTON_END_LOCATION		2
#define NO_OF_STEPS_LOCATION		3
#define STEPS_START_LOCATION		4

// Number to led additional index
#define BLANK 10
#define LETTER_C 11
#define LETTER_A 12
#define LETTER_L 13
#define LETTER_D 14
#define LETTER_O 0
#define LETTER_N 15
#define LETTER_E 16
#define LETTER_R 17
#define LETTER_S 5
#define LETTER_T 18

byte calibratedDpot = false; // If more than 0, use steps

// Calibration steps store for the dpot value and adc value (divide ADC by 3 to save space)
byte stepIndex[80][2] = {{0, 0}};

// Mapping of characters to LEDs 
byte number_to_led[19] = {
	(1<<NA) | (1<<NF) | (1<<NE) | (1<<ND) | (1<<NC) | (1<<NB), // 0
	(1<<NB) | (1<<NC), // 1
	(1<<NA) | (1<<NB) | (1<<NG) | (1<<NE) | (1<<ND), // 2
	(1<<NA) | (1<<NB) | (1<<NG) | (1<<NC) | (1<<ND), // 3
	(1<<NF) | (1<<NG) | (1<<NB) | (1<<NC), // 4
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NC) | (1<<ND), // 5
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NE) | (1<<ND) | (1<<NC), // 6
	(1<<NA) | (1<<NB) | (1<<NC), // 7
	(1<<NA) | (1<<NB) | (1<<NC) | (1<<ND) | (1<<NE) | (1<<NF) | (1<<NG), // 8
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NB) | (1<<NC), // 9
	0, // 10 (Blank)
	(1<<NA) | (1<<NF) | (1<<NE) | (1<<ND), // C
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NB) | (1<<NE) | (1<<NC), // A
	(1<<NF) | (1<<NE) | (1<<ND), // L
	(1<<NG) | (1<<NE) | (1<<ND) | (1<<NC) | (1<<NB), // D
	(1<<NE) | (1<<NG) | (1<<NC), // N
	(1<<NA) | (1<<NF) | (1<<NG) | (1<<NE) | (1<<ND), // E
	(1<<NE) | (1<<NG), // R
	(1<<NF) | (1<<NE) | (1<<NG) | (1<<ND), // T
};

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// ADC Read
int analog_read(uint8_t pin, uint8_t reference) {
	uint8_t low, high;
	
	// Does the reference need to be changed?
	if ((ADMUX & reference) == 0) {
		cbi(ADCSRA, ADEN); // Turn off ADC
		ADMUX = (reference) | (pin & 0x3f); // Set the analog pin and reference
		sbi(ADCSRA, ADEN); // Turn on ADC
	}
	else {
		// Set the analog pin without turning off the ADC
		ADMUX = (reference) | (pin & 0x3f);
	}
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// ADC Read with averaging of 5 readings
int analog_read_average(uint8_t pin, uint8_t reference) {
	int adcreading = 0;
	for (int x = 0; x < 5; x++) {
		adcreading = adcreading + analog_read(pin, reference);
	}
	adcreading = adcreading / 5;
	
	return adcreading;
}

// Setup the ATtiny
void setup(void) {
	// Initialise the SPI
	DDRA |= (1<<PA4); // SPI CLK
	DDRA |= (1<<PA5); // SPI DO
	//DDRA &= ~(1<<PA6); // SPI DI
	//PORTA |= (1<<PA6); // SPI DI
	DDRB |= (1<<latch); // Latch
	DDRB |= (1<<dpot_latch); // Dpot CS
	
	PORTB |= (1<<latch); // Latch high
	PORTB |= (1<<dpot_latch); // Dpot CS high
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS1);
	sbi(ADCSRA, ADPS0);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
	
	// Turn on ADC
	sbi(ADCSRA, ADEN);
	
	// Turn on Timer0 overflow interrupt
	sbi(TIMSK0, TOIE0);
	
	// Set Timer0 to 256 prescaler
	TCCR0B = (1<<CS01);
	
	// Pin interrupt setup for the rotary encoder
	sbi(GIMSK,PCIE0); // Enable pin change interrupt
	sbi(PCMSK0,PCINT0);
	sbi(PCMSK0,PCINT3);
	
	// Turn on Timer1 interrupt on OCR1A compare
	sbi(TIMSK1, OCIE1A);
	sbi(TCCR1B, WGM12);
	
	// Read number of steps if calibrated
	calibratedDpot = eeprom_read_byte((uint8_t*) NO_OF_STEPS_LOCATION);
	if (calibratedDpot == 255) {
		calibratedDpot = false;
	}
	else {
		int y = STEPS_START_LOCATION;
		for (int x = 0; x < calibratedDpot; x++) {
			stepIndex[x][DPOTVAL] = eeprom_read_byte((uint8_t*) y++);
			stepIndex[x][ADCVAL] = eeprom_read_byte((uint8_t*) y++);
		}
		calibratedDpot = true;
	}
	
	// Turn on interrupt
	sei();
}
